require "json"
require "net/http"
require 'fileutils'
require 'colorize'

module POEditor
  class Core

    POEDITOR_BASE_URL = "https://api.poeditor.com/v2/"
    DO_NOT_CHANGE_MANUALLY_MESSAGE = "Values below were pulled automatically from POEditor. DO NOT change them manually."

    # @return [POEditor::Configuration] The configuration for export
    attr_accessor :configuration

    # @param configuration [POEditor::Configuration]
    def initialize(configuration)
      unless configuration.is_a? Configuration
        raise POEditor::Exception.new \
          "`configuration` should be an `Configuration`"
      end
      @configuration = configuration
    end

    # Request POEditor API
    #
    # @param action [String]
    # @param api_token [String]
    # @param options [Hash{Sting => Object}]
    #
    # @return [Net::HTTPResponse] The response object of API request
    #
    # @see https://poeditor.com/api_reference/ POEditor API Reference
    def api(action, api_token, options={})
      uri = URI(POEDITOR_BASE_URL + "#{action}")
      options["api_token"] = api_token
      return Net::HTTP.post_form(uri, options)
    end

    # Pull translations
    def pull()
      UI.puts "\nExport translations"
      for language in @configuration.languages
        UI.puts "  - Exporting '#{language}'"
        content = self.export(:api_key => @configuration.api_key,
                              :project_id => @configuration.project_id,
                              :language => language,
                              :type => @configuration.type,
                              :tags => @configuration.tags,
                              :filters => @configuration.filters)
        write(language, content)

        for alias_to, alias_from in @configuration.language_alias
          if language == alias_from
            write(alias_to, content)
          end
        end
      end
    end

    # Export translation for specific language
    #
    # @param api_key [String]
    # @param project_jd [String]
    # @param language [String]
    # @param type [String]
    # @param tags [Array<String>]
    # @param filters [Array<String>]
    #
    # @return Downloaded translation content
    def export(api_key:, project_id:, language:, type:, tags:nil, filters:nil)
      options = {
        "id" => project_id,
        "language" => convert_to_poeditor_language(language),
        "type" => type,
        "tags" => (tags || []).join(","),
        "filters" => (filters || []).join(","),
      }
      response = self.api("projects/export", api_key, options)
      data = JSON(response.body)
      unless data["response"]["status"] == "success"
        code = data["response"]["code"]
        message = data["response"]["message"]
        raise POEditor::Exception.new "#{message} (#{code})"
      end

      download_uri = URI(data["result"]["url"])
      content = Net::HTTP.get(download_uri)
      content = content.force_encoding 'UTF-8'

      case type
      when "apple_strings"
        content.prepend("/* #{DO_NOT_CHANGE_MANUALLY_MESSAGE} */\n")
        content.gsub!(/(%(\d+\$)?)s/, '\1@')  # %s -> %@
      when "android_strings"
        content = content.gsub(/<resources>/, "<!-- #{DO_NOT_CHANGE_MANUALLY_MESSAGE} -->\n\\0")
        content.gsub!(/(%(\d+\$)?)@/, '\1s')  # %@ -> %s
        content.gsub!(/\.\.\./, '…')  # ... -> … (three dots -> single character for elipsis)
      end

      unless content.end_with? "\n"
        content += "\n"
      end
      return content
    end

    def convert_to_poeditor_language(language)
      language = language.downcase
      android_region_language_regexp =  /(?<=[a-z]{2}-)[r](?=[a-z]{2})/i
      chinese_regions_regexp = /(zh-)\K(hans|hant)/i
      if language =~ android_region_language_regexp
        return language.gsub(android_region_language_regexp, "")
      elsif language =~ chinese_regions_regexp
        return language.gsub(chinese_regions_regexp, {'hans' => 'cn', 'hant' => 'tw'})
      end
      language
    end

    # Write translation file
    def write(language, content)
      if content.to_s.strip.empty?
        UI.puts "      #{"\xe2\x9c\x95".red} Ignoring language: #{language} because there are no any translations."
        return
      end
      paths = paths_for_language(language)
      paths.each { | path | write_translation_to_path(path, content)}
    end

    def paths_for_language(language)
      if @configuration.path_copy[language]
        language = convert_to_android_bcp_47(@configuration.path_copy[language], language)
        [@configuration.path_copy[language], @configuration.path.gsub("{LANGUAGE}", language)]
      elsif @configuration.path_replace[language]
        [@configuration.path_replace[language]]
      else
        language = convert_to_android_bcp_47(@configuration.path, language)
        [@configuration.path.gsub("{LANGUAGE}", language)]
      end
    end

    def write_translation_to_path(path, content)
      if path.to_s.empty?
        raise POEditor::Exception.new "#{path} doesn't exist"
      end
      dirname = File.dirname(path)
      unless File.directory?(dirname)
        FileUtils.mkdir_p(dirname)
      end
      File.write(path, content)
      UI.puts "      #{"\xe2\x9c\x93".green} Saved at '#{path}'"
    end

    def convert_to_android_bcp_47(translation_path, language)
        # added to follow b+ convention of BCP-47 on Android
        if translation_path.include? "b+"
          return language.gsub("-", "+")
        end
        return language
    end

  end
end
